<?php

namespace IMATHUZH\Qfq\Core\Form\FormElement;

use IMATHUZH\Qfq\Core\Form\Form;
use IMATHUZH\Qfq\Core\Database\DatabaseManager;
use IMATHUZH\Qfq\Core\Helper\Path;
use UserFormException;

/**
 * Class UploadFormElementPublication
 *
 * Handles the import of publication data from an uploaded file (e.g., BibTeX).
 * It parses the BibTeX entries and inserts them into a database.
 *
 * Example BibTeX entry:
 * @article{smith2020,
 *       title = {A Study on Something},
 *       author = {John Smith},
 *       year = {2020}
 *   }
 * This parser extracts:
 *   - type: "article"
 *   - citeKey: "smith2020"
 *   - fields: An associative array of additional fields (title, author, year, etc.)
 */
class UploadFormElementPublication {

    // Core properties inherited from higher level classes.
    protected $attributes;    // Array of form attributes (including FILE_TYPE_KEY)
    protected $form;          // Form object (if available)
    protected $value;         // The file path or raw value of the upload
    protected $targetTable;   // The target database table (e.g., TABLE_PUBLICATION)

    /**
     * Constructor: Initializes the publication upload element.
     *
     * @param array $attributes Array of form-specific attributes.
     * @param Form|null $form Optional Form object.
     * @param string $value The file path or value.
     */
    public function __construct(array $attributes, ?Form $form = null, string $value = '') {
        $this->attributes = $attributes;
        $this->form = $form;
        $this->value = $value;

        // Set target table from attributes or use default TABLE_PUBLICATION.
        $this->targetTable = $attributes[FIELD_TARGETTABLE] ?? TABLE_PUBLICATION;

        //  set ordered keys
        $this->orderedKeys = [
            'citeKey',
            'author',
            'title',
            'journal',
            'booktitle',
            'publisher',
            'address',
            'chapter',
            'edition',
            'editor',
            'isbn',
            'month',
            'year',
            'volume',
            'number',
            'pages',
            'series',
            'abstract',
            'doi',
            'eprint',
            'keywords',
            'url',
            'note',
            'organization',
            'school',
            'institution'
        ];

        // Initialize the database connection.
        $this->db = DatabaseManager::getInstance()->getDataDb();

        // Immediately process the file if the file type indicates a BibTeX file.
        $this->handleFileType();
    }

    /**
     * Sets the value (e.g., file path) for this upload element.
     *
     * @param string $value New file path.
     */
    public function setValue(string $value): void {
        $this->value = $value;
    }

    /**
     * Checks the file type parameter and processes the upload accordingly.
     *
     * This centralizes file type handling inside the publication class.
     * If the attribute FILE_TYPE_KEY equals FILE_TYPE_BIB, processUpload() is called.
     */
    private function handleFileType(): void {
        switch ($this->attributes[FILE_TYPE_KEY]??'') {
            case FILE_TYPE_BIB:
                $this->processUpload();
                break;
            // Future file types can be handled here.
            default:
                // Optionally handle unknown or generic file types.
                break;
        }
    }

    /**
     * Processes the uploaded file:
     *  1. The absolute file path.
     *  2. Checks if the file exists and is readable.
     *  3. Reads the file content.
     *  4. Parses the BibTeX entries.
     *  5. Inserts each entry into the database.
     *  6. Deletes the file after processing.
     *
     * Example: After processing, an entry like @article{smith2020, ...}
     * will be extracted and inserted into the designated database table.
     */
    public function processUpload(): void {
        $absolutePath = Path::absoluteApp($this->value); // Get the absolute file path.
        if (file_exists($absolutePath) && is_readable($absolutePath)) {
            $fileContent = file_get_contents($absolutePath);  // Read the file content.
            $entries = $this->parseBibtex($fileContent);        // Parse BibTeX entries.
            foreach ($entries as $pubEntry) {
                $this->insertPublication($pubEntry);            // Insert each entry.
            }
            // Delete the file after processing.
            if (!unlink($absolutePath)) {

                throw new UserFormException(json_encode([
                    ERROR_MESSAGE_TO_USER => 'The file could not be deleted. Please try again or contact support.',
                    ERROR_MESSAGE_TO_DEVELOPER => 'Error deleting the file after processing all BibTeX entries. Check file permissions and review server logs for more details.',
                    ERROR_MESSAGE_HTTP_STATUS => 'HTTP/1.0 500 Internal Server Error'
                ]), ERROR_MISSING_REQUIRED_PARAMETER);
            }
        }
    }

    /**
     * Parses the BibTeX text into an associative array of entries.
     *
     * Uses the regex:
     *   '/@(\w+)\s*\{\s*([^,]+),\s*(.*?)\s*\}(?=\s*@|\s*$)/s'
     *
     *  Brief explanation:
     *    - **(\w+)**: Captures the entry type (e.g., "article").
     *    - **\{\s*([^,]+),**: Matches the opening brace and captures the citation key (e.g., "smith2020") up to the first comma.
     *    - **(.*?)\s*\}**: Non-greedily captures the fields content (all key-value pairs) until the closing brace.
     *    - **(?=\s*@|\s*$)**: Ensures the match ends right before the next entry or at the end of the text.
     *
     * Regex Breakdown:
     *   - Group 1: Captures the entry type (e.g., "article").
     *   - Group 2: Captures the citation key (e.g., "smith2020").
     *   - Group 3: Captures the fields content.
     *
     * Example BibTeX text:
     *
     * @article{smith2020,
     *     title = {A Brief Title},
     *     author = {Smith, John},
     *     year = {2020}
     *   }
     *
     * In this example:
     *   - Group 1 will capture "article"
     *   - Group 2 will capture "smith2020"
     *   - Group 3 will capture:
     *       "title = {A Brief Title},
     *        author = {Smith, John},
     *        year = {2020}"
     *
     * Example structure of $matches (after applying preg_match_all):
     *
     *   Array
     *   (
     *       [0] => Array
     *           (
     *               [0] => "@article{smith2020,
     *                        title = {A Brief Title},
     *                        author = {Smith, John},
     *                        year = {2020}"
     *               [1] => "article"
     *               [2] => "smith2020"
     *               [3] => "title = {A Brief Title},
     *                        author = {Smith, John},
     *                        year = {2020}"
     *           )
     *   )
     *
     * Returns an array of entries, each being an associative array of field values.
     *
     * @param string $bibtexText The complete BibTeX text.
     * @return array Parsed entries.
     */
    private function parseBibtex(string $bibtexText): array {
        $entries = [];
        preg_match_all(
            '/@(\w+)\s*\{\s*([^,]+),\s*(.*?)\s*\}(?=\s*@|\s*$)/s',
            $bibtexText,
            $matches,
            PREG_SET_ORDER
        );
        foreach ($matches as $match) {
            // Parse the fields content directly from Group 3.
            $fields = $this->parseFields($match[3]);
            // Normalize and store the entry type (Group 1) as a lowercase value.
            $fields[FIELD_TYPE] = strtolower($match[1]);
            // Store the citation key (Group 2) without modifications.
            $fields[FIELD_CITEKEY] = $match[2];
            // Append the fully processed entry to the entries array.
            $entries[] = $fields;
        }
        return $entries;
    }

    /**
     * Parses the fields from the BibTeX fields text.
     *
     * This function extracts field names and their corresponding values from a string
     * containing BibTeX field definitions. It also normalizes the field names (e.g., "Titel" becomes "titel")
     * and correctly extracts values that are wrapped in curly braces, double quotes, or given as numeric values.
     *
     * When processing the fields, the function uses PHP's trim() function to remove any leading or trailing whitespace.
     * This ensures that any extra spaces accidentally present in the input do not affect the keys or values.
     *
     * For example:
     *
     *   // Before using trim()
     *   $dirtyKey   = "  Titel  ";  // Contains extra spaces at the beginning and end.
     *   $dirtyValue = "  A Great Book  "; // Also contains extra spaces.
     *
     *   // After using trim()
     *   $cleanKey   = trim($dirtyKey);    // Results in "Titel"
     *   $cleanValue = trim($dirtyValue);   // Results in "A Great Book"
     *
     * These clean values are then normalized (the key is converted to lowercase) and stored in the result array.
     *
     * Example of a complete BibTeX publication entry (as processed by parseBibtex()):
     *
     * @article{doe2021,
     *     Titel = {A Great Book},
     *     Autor = "John Doe",
     *     Jahr = 1963,
     *     Verlag = {Fictional Press}
     *   }
     *
     * The fields text passed to this function is:
     *
     *   Titel = {A Great Book},
     *   Autor = "John Doe",
     *   Jahr = 1963,
     *   Verlag = {Fictional Press}
     *
     * Example of the "dirty" input array (i.e., keys before normalization):
     *
     *   Array
     *   (
     *       [Titel]  => A Great Book
     *       [Autor]  => John Doe
     *       [Jahr]   => 1963
     *       [Verlag] => Fictional Press
     *   )
     *
     * Resulting array after normalization (keys converted to lowercase and values trimmed):
     *
     *   Array
     *   (
     *       [titel]  => A Great Book
     *       [autor]  => John Doe
     *       [jahr]   => 1963
     *       [verlag] => Fictional Press
     *   )
     *
     * Additional examples for individual fields:
     *
     * 1. Field with a value in curly braces:
     *    - Input:  title = {A Great Book},
     *    - Result: Key "title" with value "A Great Book"
     *
     * 2. Field with a value in double quotes:
     *    - Input:  author = "John Doe",
     *    - Result: Key "author" with value "John Doe"
     *
     * 3. Field with a numeric value:
     *    - Input:  year = 1963,
     *    - Result: Key "year" with value "1963"
     *
     * Regex used :
     *   '/(\w+)\s*=\s*(?:\{([^}]+)\}|\"([^\"]+)\"|(\d+))\s*(?:,|$)/s'
     *
     *  Brief explanation:
     *  - **(\w+)**: Captures the field name (e.g., "Titel").
     *  - **\s*=\s***: Matches the equal sign, allowing for optional whitespace on either side.
     *  - **(?:\{([^}]+)\}|\"([^\"]+)\"|(\d+))**: Matches the field value, which can be:
     *      - Enclosed in curly braces (captured in group 2),
     *      - Enclosed in double quotes (captured in group 3), or
     *      - A numeric value (captured in group 4).
     *  - **\s*(?:,|$)**: Matches any trailing whitespace, followed by a comma or the end of the string.
     *
     * @param string $fieldsText The text containing all the fields of a BibTeX entry.
     * @return array Associative array where keys (in lowercase) are field names and values are the extracted contents.
     */
    private function parseFields(string $fieldsText): array {
        preg_match_all(
            '/(\w+)\s*=\s*(?:\{([^}]+)\}|\"([^\"]+)\"|(\d+))\s*(?:,|$)/s',
            $fieldsText,
            $matches,
            PREG_SET_ORDER
        );
        return array_reduce($matches, function ($result, $field) {
            // Normalize key and choose the first non-empty value (prefer {} over "" over numeric)
            $key = strtolower(trim($field[1]));
            $value = trim($field[2] ?? $field[3] ?? $field[4] ?? '');
            $result[$key] = $value;
            return $result;
        }, []);
    }


    /**
     * Inserts a BibTeX entry into the Publication table.
     *
     * Retrieves the owner ID from attributes and uses the parsed BibTeX entry fields.
     * Throws an exception if required parameters (owner ID, target table) are missing.
     *
     * @param array $pubEntry The parsed BibTeX entry.
     * @throws UserFormException if mandatory parameters are missing.
     */
    private function insertPublication(array $pubEntry): void {


        // Verify that the owner ID is provided.
        if (trim($this->attributes[OWNER_ID_KEY] ?? '') === '') {
            throw new UserFormException(json_encode([
                ERROR_MESSAGE_TO_USER => 'Owner ID (Person ID) is missing.',
                ERROR_MESSAGE_TO_DEVELOPER => 'The owner ID (pId) must be set in the form as parameter.',
                ERROR_MESSAGE_HTTP_STATUS => 'HTTP/1.0 400 Bad Request'
            ]), ERROR_MISSING_REQUIRED_PARAMETER);
        }
        // Verify that the target database table is provided.
        if (trim($this->attributes[FIELD_TARGETTABLE] ?? '') === '') {
            throw new UserFormException(json_encode([
                ERROR_MESSAGE_TO_USER => 'The target Database table field is missing.',
                ERROR_MESSAGE_TO_DEVELOPER => 'The target Database table field (targetTable) must be set in the form as parameter.',
                ERROR_MESSAGE_HTTP_STATUS => 'HTTP/1.0 400 Bad Request'
            ]), ERROR_MISSING_REQUIRED_PARAMETER);
        }

        $pIdOwner = $this->attributes[OWNER_ID_KEY];

        // Aufbau des Parameters-Arrays:
        // 1. pIdOwner
        // 2. status (hier Standardwert 'new')
        // 3. anschließend die Werte aus $orderedKeys in exakt dieser Reihenfolge.
        $params = [];
        $params[] = $pIdOwner;
        $params[] = $pubEntry['citeKey'] ?? '';
        foreach ($this->orderedKeys as $key) {
            $params[] = ($key === 'year') ? ($pubEntry[$key] ?? 0) : ($pubEntry[$key] ?? '');
        }

        $sql = "INSERT INTO {$this->attributes[FIELD_TARGETTABLE]} (
            pIdOwner, status, citationKey, author, title, detail, bookTitle, publisher, address,
            chapter, edition, editor, isbn, month, pubYear, volume, number, pages, series,
            abstractchapter, dxDoi, eprint, keywords, url, note, organization, school, institution
        ) VALUES (
            ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?
        )";

        $this->db->sql($sql, ROW_REGULAR, $params);
    }
}